#!/usr/bin/ruby
#  encoding: UTF-8
#
# Check RabbitMQ Queue Messages
# ===
#
# DESCRIPTION:
# This plugin checks the number of messages queued on the RabbitMQ server in a specific queues
#
# PLATFORMS:
#   Linux, BSD, Solaris
#
# DEPENDENCIES:
#   RabbitMQ rabbitmq_management plugin
#   gem: sensu-plugin
#   gem: carrot-top
#
# LICENSE:
# Copyright 2012 Evan Hazlett <ejhazlett@gmail.com>
#
# Released under the same terms as Sensu (the MIT license); see LICENSE
# for details.

require 'sensu-plugin/check/cli'
require 'bunny'

# main plugin class
class CheckAMQPStatus < Sensu::Plugin::Check::CLI
  option :host,
         description: 'RabbitMQ host',
         long: '--host HOST',
         required: true,
         default: 'localhost'

  option :port,
         description: 'RabbitMQ port',
         long: '--port PORT',
         proc: proc(&:to_i),
         default: 5_672

  option :vhost,
         description: 'RabbitMQ vhost',
         short: '-v',
         long: '--vhost VHOST',
         default: ''

  option :user,
         description: 'RabbitMQ user',
         long: '--user USER',
         default: 'guest'

  option :password,
         description: 'RabbitMQ password',
         long: '--password PASSWORD',
         default: 'guest'

  option :queue,
         description: 'RabbitMQ status queue to monitor',
         long: '--queue queue_name',
         required: true,
         default: 'status'

  def connect
    conn = Bunny.new("amqp://#{config[:user]}:#{config[:password]}@#{config[:host]}:#{config[:port]}#{config[:vhost]}")
    conn.start
  end

  def run
    # Connect to RabbitMQ
    rabbitmq = connect
    channel = rabbitmq.create_channel
    q = channel.queue(config[:queue], exclusive: false, durable: true)

    # Check if there are any messages to be consumed
    count = q.message_count
    if count == 0
      critical "critical: No alive message found.  #{config[:queue]} queue is empty."
    end

    # Consume the messages one at a time since this is not meant to be a persistent process
    until count < 1
      q.pop(block: true, manual_ack: false) do |_delivery_info, properties, payload|
        if payload.nil?
          critical "critical: Payload empty even though there was supposed to be a message. Message properties are #{properties.inspect}"
        else
          puts "Received #{payload}, message properties are #{properties.inspect}"
        end
        count -= 1
      end
    end

    # Assuming we got this far, exit cleanly
    ok
  end
end
